//
//  ViewController.m
//  LinkkitDemo
//
//  Created by 朱永利 on 2018/11/22.
//  Copyright © 2018年 朱永利. All rights reserved.
//

#import "EntryTableViewController.h"
#import <IMSHUD/UIViewController+HUD.h>
#import <IotLinkKit/IotLinkKit.h>
#import <AlinkIoTExpress/AlinkIoTExpress.h>

@interface EntryTableViewController () <UITextFieldDelegate, LinkkitChannelListener, UITextViewDelegate>
@property (weak, nonatomic) IBOutlet UITextField *textFieldProductKey;
@property (weak, nonatomic) IBOutlet UITextField *textFieldDeviceName;
@property (weak, nonatomic) IBOutlet UITextField *textFieldDeviceSecret;
@property (weak, nonatomic) IBOutlet UILabel *labelConnectState;
@property (weak, nonatomic) IBOutlet UITextField *textFieldPubTopic;
@property (weak, nonatomic) IBOutlet UITextView *textViewPubContent;
@property (weak, nonatomic) IBOutlet UITextField *textFieldSubTopic;
@property (weak, nonatomic) IBOutlet UITextView *textViewDownData;

@property (weak, nonatomic) IBOutlet UITextField *textFieldQos;
@property(nonatomic, copy) NSString *productKey;
@property(nonatomic, copy) NSString *deviceName;
@property(nonatomic, copy) NSString *deviceSecret;
@property(nonatomic, copy) NSString *appVersion;
@property(nonatomic, assign) int cleanSession;
@property(nonatomic, strong) NSDictionary * testDict;
@end

@implementation EntryTableViewController

- (IBAction)onClickFillTestData:(id)sender {
    NSDictionary *testJson = self.testDict;
    if (testJson) {
        self.productKey = [testJson valueForKey:@"productKey"];
        self.deviceName = [testJson valueForKey:@"deviceName"];
        self.deviceSecret = [testJson valueForKey:@"deviceSecret"];
        self.appVersion = [testJson valueForKey:@"appVersion"];
        self.cleanSession = [[testJson valueForKey:@"cleanSession"] intValue];
        
        if (self.productKey == nil || self.deviceName == nil
            || self.deviceSecret == nil) {
            [self ims_showHUDWithMessage:@"请先填写设备三元组！"];
        }
        
        self.textFieldProductKey.text = self.productKey;
        self.textFieldDeviceName.text = self.deviceName;
        self.textFieldDeviceSecret.text = self.deviceSecret;
        
        self.textFieldPubTopic.text = [NSString stringWithFormat:@"/%@/%@/user/update",
                                       self.productKey, self.deviceName];
        self.textFieldSubTopic.text = [NSString stringWithFormat:@"/%@/%@/user/get",
                                       self.productKey, self.deviceName];
        
        self.textViewPubContent.text = @"{\"Target\":\"6II9Fm0naqZEOVcTpMbV\",\"Service\":\"status report\",\"params\":[{\"Disconnect Setting\":\"1\"}],\"Source\":\"24:0A:C4:86:08:2E\"}";
        //@"{\"userName\":\"Hello World\"}";
    }
    else {
        [self ims_showHUDWithMessage:@"应用沙盒Document目录未找到testdata.json，请先用iTunes导入！"];
    }
}
- (IBAction)onClickSetupKit:(id)sender {
    [[LinkKitEntry sharedKit] registerChannelListener:self];
    
    self.productKey = self.textFieldProductKey.text ;
    self.deviceName = self.textFieldDeviceName.text   ;
    self.deviceSecret = self.textFieldDeviceSecret.text;
    
    LinkkitChannelConfig * channelConfig = [[LinkkitChannelConfig alloc] init];
    channelConfig.productKey = self.productKey;
    channelConfig.deviceName = self.deviceName;
    channelConfig.deviceSecret = self.deviceSecret;
    channelConfig.cleanSession = (self.cleanSession == 1);
    //channelConfig.server = @"your custom server url";
    //channelConfig.port = 1883;
    LinkkitSetupParams * setupParams = [[LinkkitSetupParams alloc] init];
    setupParams.appVersion = self.appVersion;
    setupParams.channelConfig = channelConfig;
    [[LinkKitEntry sharedKit] setup:setupParams resultBlock:^(BOOL succeeded, NSError * _Nullable error) {
        LinkkitLogDebug(@"setup error : %@", error);
        dispatch_async(dispatch_get_main_queue(), ^{
            [self ims_showHUDWithMessage:[NSString stringWithFormat:@"Linkkit 初始化 : %@",
                                          succeeded ? @"成功" : @"失败"]];
        });
    }];
}
- (IBAction)onClickDestroyKit:(id)sender {
    [[LinkKitEntry sharedKit] destroy:^(BOOL succeeded, NSError * _Nullable error) {
        LinkkitLogDebug(@"kit destroy error : %@", error);
        dispatch_async(dispatch_get_main_queue(), ^{
            [self ims_showHUDWithMessage:[NSString stringWithFormat:@"Linkkit 去初始化 : %@",
                                          succeeded ? @"成功" : @"失败"]];
        });
    }];
}


static NSString * sUptopic  ;
static NSString * sUpcontent ;
static NSInteger sUpQos ;

- (IBAction)onClickPublish:(id)sender {
#if 1
    NSString * topic = self.textFieldPubTopic.text;
    NSString * content = self.textViewPubContent.text;
    NSInteger iQos = [self.textFieldQos.text integerValue];
    
    [[LinkKitEntry sharedKit] publish:topic
                                data:[content dataUsingEncoding:NSUTF8StringEncoding]
                                 qos:(int)iQos
                         resultBlock:^(BOOL succeeded, NSError * _Nullable error) {
                             LinkkitLogDebug(@"kit publish error : %@", error);
                             dispatch_async(dispatch_get_main_queue(), ^{
                                 [self ims_showHUDWithMessage:[NSString stringWithFormat:@"Linkkit 上行数据 : %@",
                                                               succeeded ? @"成功" : @"失败"]];
                             });
                         }];
#else
    
    sUptopic = self.textFieldPubTopic.text;
    sUpcontent = self.textViewPubContent.text;
    sUpQos = [self.textFieldQos.text integerValue];
    for (int i = 0; i < 10; i++) {
        //[NSThread sleepForTimeInterval:5.f];
        NSThread * thread = [[NSThread alloc] initWithTarget:self
                                                    selector:@selector(repeatUploadThreadEntry:)
                                                      object:@(i)] ;
        [thread start];
    }
#endif
}

-(void)repeatUploadThreadEntry : (int)idx {
    float startSleep = arc4random_uniform((uint32_t) 10)*1.f;
    [NSThread sleepForTimeInterval:startSleep];
    for (int j = 0; j < 300; j++) {
//        _bindRepeatCounter++;
//        AKLogDebug(@"bind repeat counter : %d", _bindRepeatCounter);
//        dispatch_async(dispatch_get_main_queue(), ^{
//            [_tipsLabel setText:[NSString stringWithFormat:@"绑定/解绑 %d 次", _bindRepeatCounter]];
//        });
//        int idx = arc4random_uniform((uint32_t) 10);
//        if (idx %2 == 0) {
//            [self doBind];
//        } else {
//            [self doUnbind];
//        }
        
//        [[LinkKitEntry sharedKit] upload:topic
//                                    data:[content dataUsingEncoding:NSUTF8StringEncoding]
//                                     qos:(int)iQos
//                             resultBlock:^(BOOL succeeded, NSError * _Nullable error) {
//                                 NSLog(@"%@", [NSString stringWithFormat:@"Linkkit 上行数据 : %@", succeeded ? @"成功" : @"失败"]);
////                                 dispatch_async(dispatch_get_main_queue(), ^{
////                                     [self ims_showHUDWithMessage:[NSString stringWithFormat:@"Linkkit 上行数据 : %@",
////                                                                   succeeded ? @"成功" : @"失败"]];
////                                 });
//                             }];
        [[LKIoTExpress sharedInstance] uploadData:sUptopic
                                             data:[sUpcontent dataUsingEncoding:NSUTF8StringEncoding]
                                              qos:(int)sUpQos
                                         complete:^(NSError * _Nonnull err) {
            NSLog(@"%@", [NSString stringWithFormat:@"Linkkit 上行数据 : %@", err == nil ? @"成功" : @"失败"]);
        }];
        float sleepSec = arc4random_uniform((uint32_t) 20) * 1.f;
        [NSThread sleepForTimeInterval:sleepSec];
        if (idx == 9 && j == 299) {
            //self.isBindRepeat = NO;
        }
    }
}

- (IBAction)onClickUnsubscribe:(id)sender {
    
    NSString * topic = self.textFieldSubTopic.text;
    [[LinkKitEntry sharedKit] unsubscribe:topic
                            resultBlock:^(BOOL succeeded, NSError * _Nullable error) {
                                LinkkitLogDebug(@"kit unsubscribe error : %@", error);
                                dispatch_async(dispatch_get_main_queue(), ^{
                                    [self ims_showHUDWithMessage:[NSString stringWithFormat:@"Linkkit 取消订阅topic : %@，",
                                                                  succeeded ? @"成功" : @"失败"]];
                                });
                            }];
}

- (IBAction)onClickSubscribe:(id)sender {
    NSString * topic = self.textFieldSubTopic.text;
    [[LinkKitEntry sharedKit] subscribe:topic
                            resultBlock:^(BOOL succeeded, NSError * _Nullable error) {
                                LinkkitLogDebug(@"kit subscribe error : %@", error);
                                dispatch_async(dispatch_get_main_queue(), ^{
                                    [self ims_showHUDWithMessage:[NSString stringWithFormat:@"Linkkit 订阅topic : %@，",
                                                                  succeeded ? @"成功" : @"失败"]];
                                });
    }];
}

- (IBAction)onClickClearDown:(id)sender {
    self.textViewDownData.text = @"";
}


- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.textFieldQos.delegate = self;
    self.textFieldPubTopic.delegate = self;
    self.textFieldSubTopic.delegate = self;
    self.textFieldDeviceName.delegate = self;
    self.textFieldProductKey.delegate = self;
    self.textFieldDeviceSecret.delegate = self;
    self.textViewPubContent.delegate = self;
    self.textViewDownData.delegate = self;
    
    
    UITapGestureRecognizer *gestureRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self
                                                                                        action:@selector(hideKeyboard)];
    [self.tableView addGestureRecognizer:gestureRecognizer];
    
    [LinkKitEntry sharedKit];
    // Do any additional setup after loading the view, typically from a nib.
    
    // 得到Documents路径
    NSString *docPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) lastObject];
    
    // 创建一个文件路径
    NSString *filePath = [docPath stringByAppendingPathComponent:@"testdata.json"];
    
    NSFileManager* fm = [NSFileManager defaultManager];
    if ([fm fileExistsAtPath:filePath]) {
        NSData * testData = [NSData dataWithContentsOfFile:filePath];
        NSDictionary * testJson = [NSJSONSerialization JSONObjectWithData:testData
                                                                  options:0
                                                                    error:nil];
        self.testDict = testJson;
    } else {
//        NSString * scriptPath = [[NSBundle mainBundle] pathForResource:@"testdata" ofType:@"json"];
//        NSData * testData = [NSData dataWithContentsOfFile:scriptPath];
//        if (testData) {
//            NSDictionary * testJson = [NSJSONSerialization JSONObjectWithData:testData
//                                                                      options:0
//                                                                        error:nil];
//            if ([testJson count]) {
//                self.testDict = testJson;
//                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(2 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
//                    [self onClickFillTestData:nil];
//                    [self onClickSetupKit:nil];
//                });
//            }
//        }
    }
    
}


- (void)onConnectStateChange:(nonnull NSString *)connectId state:(LinkkitChannelConnectState)state {
    NSString * connectTip = nil;
    if (state == LinkkitChannelStateConnected) {
        connectTip = @"已连接";
    } else if (state == LinkkitChannelStateDisconnected) {
        connectTip = @"未连接";
    } else {
        connectTip = @"连接中";
    }
    dispatch_async(dispatch_get_main_queue(), ^{
        self.labelConnectState.text = connectTip;
    });
}

- (void)onNotify:(nonnull NSString *)connectId topic:(nonnull NSString *)topic data:(id _Nullable)data {
    NSString * downData = [NSString stringWithFormat:@"收到下推，topic : %@ \r\n", topic];
    downData = [downData stringByAppendingString:[NSString stringWithFormat:@"\r\n数据 : %@", data]];
    
    LinkkitLogDebug(@"kit recv  topic : %@", topic);
    
    dispatch_async(dispatch_get_main_queue(), ^{
        self.textViewDownData.text = downData;
    });
}

- (BOOL)shouldHandle:(nonnull NSString *)connectId topic:(nonnull NSString *)topic {
    return YES;
}


#pragma mark - text field delegate
- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField {
    return YES;
}

- (BOOL)textFieldShouldEndEditing:(UITextField *)textField {
    return YES;
}

- (BOOL)textFieldShouldClear:(UITextField *)textField {
    return YES;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    return YES;
}

- (void)hideKeyboard {
    [self.view endEditing:YES];
}
@end
