//
//  GatewayViewController.m
//  LinkkitDemo
//
//  Created by 朱永利 on 2018/11/23.
//  Copyright © 2018年 朱永利. All rights reserved.
//

#import "GatewayViewController.h"
#import "NIDropDown.h"
#import <IMSHUD/UIViewController+HUD.h>
#import <IotLinkKit/IotLinkKit.h>

@interface KitSubDeviceSigner : NSObject<ILKSubDeviceSigner>

/**
 子设备签名信息时使用的签名方法
 可选值: "hmacmd5 or hmacsha1 or hmacsha256 or sha256",
 */
@property(nonatomic, copy, readwrite) NSString * signMethod;


/**
 计算sign时的 salt，可以取 mac，也可以是随机字串，由用户自己确定。
 */
@property(nonatomic, copy, readwrite) NSString * clientId;

/**
 子设备签名信息，
 对于三元组设备 子设备签名规则如下：
 clientId 客户端标识，可以取 mac，也可以是随机字串，由设备端确定,对于蓝牙设备，signMethod 指定为 sha256
 假定：clientId=123, deviceName=test, productKey=123,deviceSecret=secret. signMethod:"sha256",
 则 sign 计算规则如下
 sign = sha256(clientId123deviceNametestdeviceSecretsecretproductKey123)
 */
@property(nonatomic, copy, readwrite) NSString * signValue;
@end

@implementation KitSubDeviceSigner

@synthesize clientId;

@synthesize signMethod;

@synthesize signValue;

@end

@interface GatewayViewController () <UITextFieldDelegate, UITextViewDelegate,
                                        NIDropDownDelegate, ILKSubDeviceDelegate, UIGestureRecognizerDelegate>
@property (weak, nonatomic) IBOutlet UITextField *textFieldPubTopic;
@property (weak, nonatomic) IBOutlet UITextView *textViewPubContent;
@property (weak, nonatomic) IBOutlet UITextField *textFieldSubTopic;
@property (weak, nonatomic) IBOutlet UITextView *textViewDownData;
@property (weak, nonatomic) IBOutlet UILabel *lableSubState;
@property (weak, nonatomic) IBOutlet UITextField *textFieldQos;

@property(nonatomic, strong) NIDropDown *dropDown;
@property(nonatomic, strong) NSArray *subDevices;
@property(nonatomic, strong) NSArray *subDevicesReg;
@property(nonatomic, assign) LinkkitDeviceAuth * selSub;

@property(nonatomic, strong) id<LinkkitGateway> gatewayInterface;
@property(nonatomic, strong) id<ILKSubDeviceChannel> subChannel;
@end

@implementation GatewayViewController

- (IBAction)onClickBatchReg:(id)sender {
    if ([self.subDevices count] == 0) {
        [self ims_showHUDWithMessage:@"沙盒目录中 testdata.json 没有填入子设备信息"];
        return;
    }
    
    if (self.gatewayInterface == nil) {
        [self ims_showHUDWithMessage:@"请先返回首页初始化Linkkit！"];
        return;
    }
    
    NSMutableArray * subBases = @[].mutableCopy;
    for (NSDictionary * subinfo in _subDevices) {
        LinkkitDeviceBase * base = [[LinkkitDeviceBase alloc] init];
        base.productKey = [subinfo valueForKey:@"productKey"];
        base.deviceName = [subinfo valueForKey:@"deviceName"];
        [subBases addObject:base];
    }
    [self.gatewayInterface subDeviceRegisterBatch:subBases
                                      resultBlock:^(NSArray * _Nullable result, NSError * _Nullable error) {
                                          LinkkitLogDebug(@"subDeviceRegisterBatch error : %@", error);
                                          self.subDevicesReg = result;
                                          dispatch_async(dispatch_get_main_queue(), ^{
                                              [self ims_showHUDWithMessage:[NSString stringWithFormat:@"成功注册子设备个数 : %d",
                                                                            (int)self.subDevicesReg.count]];
                                          });
    }];
}

- (void)viewWillAppear:(BOOL)animated {
    NSLog(@"Gateway view controller viewWillAppear");
    self.gatewayInterface = [[LinkKitEntry sharedKit] gatewayInterface];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.textFieldQos.delegate = self;
    self.textFieldPubTopic.delegate = self;
    self.textFieldSubTopic.delegate = self;
    self.textViewPubContent.delegate = self;
    self.textViewDownData.delegate = self;
    
    UITapGestureRecognizer *gestureRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self
                                                                                        action:@selector(hideKeyboard)];
    gestureRecognizer.delegate = self;
    [self.tableView addGestureRecognizer:gestureRecognizer];
    
    self.gatewayInterface = [[LinkKitEntry sharedKit] gatewayInterface];
    
    // Do any additional setup after loading the view.
    // 得到Documents路径
    NSString *docPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) lastObject];
    
    // 创建一个文件路径
    NSString *filePath = [docPath stringByAppendingPathComponent:@"testdata.json"];
    
    NSFileManager* fm=[NSFileManager defaultManager];
    if ([fm fileExistsAtPath:filePath]) {
        NSData * testData = [NSData dataWithContentsOfFile:filePath];
        NSDictionary * testJson = [NSJSONSerialization JSONObjectWithData:testData
                                                                  options:0
                                                                    error:nil];
        self.subDevices = [testJson valueForKey:@"subDevices"];
    }
}


- (IBAction)onClickUpload:(id)sender {
    
    if (self.gatewayInterface == nil) {
        [self ims_showHUDWithMessage:@"请先返回首页初始化Linkkit！"];
        return;
    }
    
    if (self.subChannel == nil) {
        [self ims_showHUDWithMessage:@"请添加子设备并上线"];
        return;
    }
    __weak typeof (self) weakSelf = self;
    
    NSString * topic = self.textFieldPubTopic.text;
    NSData * upData = [self.textViewPubContent.text dataUsingEncoding:NSUTF8StringEncoding];
    int qos = [self.textFieldQos.text intValue];
    [self.subChannel publish:topic data:upData qos:qos
                resultBlock:^(BOOL succeeded, NSError * _Nullable error) {
                    LinkkitLogDebug(@"subDevice upload error : %@", error);
                    dispatch_async(dispatch_get_main_queue(), ^{
                        [weakSelf ims_showHUDWithMessage:[NSString stringWithFormat:@"子设备数据上行 : %@", error ? @"失败":@"成功"]];
                    });
    }];
}

- (IBAction)onClickUnsubscribe:(id)sender {
    if (self.gatewayInterface == nil) {
        [self ims_showHUDWithMessage:@"请先返回首页初始化Linkkit！"];
        return;
    }
    
    
    if (self.subChannel == nil) {
        [self ims_showHUDWithMessage:@"请添加子设备并上线"];
        return;
    }
    
    __weak typeof (self) weakSelf = self;
    
    NSString * topic = self.textFieldSubTopic.text;
    [self.subChannel unsubscribe:topic
                   resultBlock:^(BOOL succeeded, NSError * _Nullable error) {
                       LinkkitLogDebug(@"subDevice unsubscribe error : %@", error);
                       dispatch_async(dispatch_get_main_queue(), ^{
                           [weakSelf ims_showHUDWithMessage:[NSString stringWithFormat:@"子设备取消订阅 : %@", error ? @"失败":@"成功"]];
                       });
                   }];
}


- (IBAction)onClickSubscribe:(id)sender {
    
    if (self.gatewayInterface == nil) {
        [self ims_showHUDWithMessage:@"请先返回首页初始化Linkkit！"];
        return;
    }
    
    
    if (self.subChannel == nil) {
        [self ims_showHUDWithMessage:@"请添加子设备并上线"];
        return;
    }
    
    __weak typeof (self) weakSelf = self;
    
    NSString * topic = self.textFieldSubTopic.text;
    [self.subChannel subscribe:topic
                   resultBlock:^(BOOL succeeded, NSError * _Nullable error) {
                       LinkkitLogDebug(@"subDevice subscribe error : %@", error);
                       dispatch_async(dispatch_get_main_queue(), ^{
                           [weakSelf ims_showHUDWithMessage:[NSString stringWithFormat:@"子设备订阅 : %@", error ? @"失败":@"成功"]];
                       });
    }];
}


- (IBAction)onClickLogout:(id)sender {
    
    if (self.gatewayInterface == nil) {
        [self ims_showHUDWithMessage:@"请先返回首页初始化Linkkit！"];
        return;
    }
    
    if (self.subChannel == nil) {
        [self ims_showHUDWithMessage:@"请添加子设备并上线"];
        return;
    }
    
    __weak typeof (self) weakSelf = self;
    [self.subChannel logout:^(BOOL succeeded, NSError * _Nullable error) {
        LinkkitLogDebug(@"subDevice logout error : %@", error);
        dispatch_async(dispatch_get_main_queue(), ^{
            weakSelf.lableSubState.text = @"未上线";
            [weakSelf ims_showHUDWithMessage:[NSString stringWithFormat:@"子设备下线 : %@", error ? @"失败":@"成功"]];
        });
    }];
}

- (IBAction)onClickAddToGw:(id)sender {
    
    if (self.gatewayInterface == nil) {
        [self ims_showHUDWithMessage:@"请先返回首页初始化Linkkit！"];
        return;
    }
    
    if (self.selSub == nil) {
        [self ims_showHUDWithMessage:@"请先选择子设备"];
        return;
    }
    
    __weak typeof (self) weakSelf = self;
    [self.gatewayInterface addSubDevice:self.selSub
                               delegate:self
                            resultBlock:^(id<ILKSubDeviceChannel>  _Nonnull subChannel, NSError * _Nullable error) {
                                LinkkitLogDebug(@"subDevice logout add to gateway : %@", error);
                                weakSelf.subChannel = subChannel;
                                dispatch_async(dispatch_get_main_queue(), ^{
                                    weakSelf.lableSubState.text = @"已关联至网关";
                                    [weakSelf ims_showHUDWithMessage:[NSString stringWithFormat:@"添加子设备到网关 : %@", error ? @"失败":@"成功"]];
                                });
    }];
}



/**
 一般说来，此种情况下添加子设备到网关，都是基于子设备本身烧录了设备三元组。
 子设备会使用自己的三元组按IoT规范计算一个 sign。然后将计算sign的结果以及参与计算的参数传给
 网关，网关将sign信息上传到云端，校验通过后，才能跟子设备建立topo关系，或者代理子设备上下线
 */
- (IBAction)onClickAddToGw1:(id)sender {

    if (self.gatewayInterface == nil) {
        [self ims_showHUDWithMessage:@"请先返回首页初始化Linkkit！"];
        return;
    }

    if (self.selSub == nil) {
        [self ims_showHUDWithMessage:@"请先选择子设备"];
        return;
    }
    
    
    
    KitSubDeviceSigner *signerInst = [[KitSubDeviceSigner alloc] init];
    signerInst.clientId = @"xxxxx";//请输入设备计算sign时真实的clientId
    signerInst.signMethod = @"hmacmd5 or hmacsha1 or hmacsha256 or sha256";//请根据实际计算sign时所采用的算法取值
    signerInst.signValue = @"signValue"; //sign的真实结果

    __weak typeof (self) weakSelf = self;
    LinkkitDeviceBase * devBase = [[LinkkitDeviceBase alloc] init];
    devBase.productKey = @"sub device productKey";
    devBase.deviceName = @"sub device deviceName";

    [self.gatewayInterface addSubDevice:devBase signer:signerInst
                               delegate:self resultBlock:^(id<ILKSubDeviceChannel>  _Nullable subChannel, NSError * _Nullable error) {

                                   LinkkitLogDebug(@"subDevice logout add to gateway : %@", error);
                                   weakSelf.subChannel = subChannel;
                                   dispatch_async(dispatch_get_main_queue(), ^{
                                       weakSelf.lableSubState.text = @"已关联至网关";
                                       [weakSelf ims_showHUDWithMessage:[NSString stringWithFormat:@"添加子设备到网关 : %@", error ? @"失败":@"成功"]];
                                   });
                               }];
}

- (IBAction)onClickDelFrom:(id)sender {
    
    if (self.gatewayInterface == nil) {
        [self ims_showHUDWithMessage:@"请先返回首页初始化Linkkit！"];
        return;
    }
    
    if (self.selSub == nil) {
        [self ims_showHUDWithMessage:@"请先选择子设备"];
        return;
    }
    
    __weak typeof (self) weakSelf = self;
    [self.gatewayInterface deleteSubDevice:self.selSub
                               resultBlock:^(BOOL succeeded, NSError * _Nullable error) {
                                   LinkkitLogDebug(@"subDevice delete from gateway error : %@", error);
                                   weakSelf.subChannel = nil;
                                   dispatch_async(dispatch_get_main_queue(), ^{
                                       weakSelf.lableSubState.text = @"未关联网关";
                                       [weakSelf ims_showHUDWithMessage:[NSString stringWithFormat:@"从网关删除子设备 : %@", error ? @"失败":@"成功"]];
                                   });
    }];
}

- (IBAction)onClickLogin:(id)sender {
    
    if (self.gatewayInterface == nil) {
        [self ims_showHUDWithMessage:@"请先返回首页初始化Linkkit！"];
        return;
    }
    
    if (self.subChannel == nil) {
        [self ims_showHUDWithMessage: @"请先加子设备添加至网关"];
        return;
    }
    __weak typeof (self) weakSelf = self;
    [self.subChannel login:^(BOOL succeeded, NSError * _Nullable error) {
        LinkkitLogDebug(@"subDevice login error : %@", error);
        dispatch_async(dispatch_get_main_queue(), ^{
            weakSelf.lableSubState.text = @"已上线";
            [weakSelf ims_showHUDWithMessage:[NSString stringWithFormat:@"子设备上线 : %@", error ? @"失败":@"成功"]];
        });
    }];
    //self.subChannel.stateListener = self;
}

- (IBAction)onClickSelectSub:(id)sender {
    if ([self.subDevicesReg count] == 0) {
        [self ims_showHUDWithMessage:@"请先注册子设备!"];
        return;
    }
    NSMutableArray * arr = [NSMutableArray arrayWithCapacity:self.subDevicesReg.count];
    for (LinkkitDeviceAuth * sub in self.subDevicesReg) {
        NSString * subInfo = [NSString stringWithFormat:@"productKey:%@,deviceName:%@",
                              sub.productKey, sub.deviceName];
        [arr addObject:subInfo];
    }
    
    if(self.dropDown == nil) {
        CGFloat f = 200;
        self.dropDown = [[NIDropDown alloc]showDropDown:sender :&f :arr :nil :@"down" :self.view];
        self.dropDown.delegate = self;
    }
    else {
        [self.dropDown hideDropDown:sender];
        [self relDropDown];
    }
}

- (void)relDropDown {
    self.dropDown = nil;
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

- (void)niDropDownDelegateMethod:(NIDropDown *)sender :(NSInteger)idx{
    if (idx < self.subDevicesReg.count) {
        self.selSub = [self.subDevicesReg objectAtIndex:idx];
    }
    
    __weak typeof (self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        weakSelf.textFieldPubTopic.text = [NSString stringWithFormat:@"/%@/%@/user/update",
                                       weakSelf.selSub.productKey, weakSelf.selSub.deviceName];
        weakSelf.textFieldSubTopic.text = [NSString stringWithFormat:@"/%@/%@/user/get",
                                       weakSelf.selSub.productKey, weakSelf.selSub.deviceName];
        
        weakSelf.textViewPubContent.text = @"{\"userName\":\"Hello World\"}";
    });
    [self relDropDown];
}


- (void)onConnectResult:(BOOL)success
                  error:(NSError * _Nullable)err
       subDeviceChannel:(nonnull id<ILKSubDeviceChannel>)subDeviceChannel {
    if ([self.subChannel.subDeviceProfile.deviceBaseId
         isEqualToString:subDeviceChannel.subDeviceProfile.deviceBaseId]) {
        
        //__weak typeof (self) weakSelf = self;
        dispatch_async(dispatch_get_main_queue(), ^{
            
        });
    } else {
        
    }
}

- (void)onDataPush:(nonnull NSString *)topic data:(nonnull NSData *)data {
    NSString * downData = [NSString stringWithFormat:@"收到下推，topic : %@ \r\n", topic];
    downData = [downData stringByAppendingString:[NSString stringWithFormat:@"\r\n数据 : %@", data]];
    
    LinkkitLogDebug(@"subDevice recv topic : %@", topic);
    dispatch_async(dispatch_get_main_queue(), ^{
        self.textViewDownData.text = downData;
    });
}

#pragma mark - text field delegate
- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField {
    return YES;
}

- (BOOL)textFieldShouldEndEditing:(UITextField *)textField {
    return YES;
}

- (BOOL)textFieldShouldClear:(UITextField *)textField {
    return YES;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    return YES;
}

- (void)hideKeyboard {
    [self.view endEditing:YES];
}


- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch
{
    if (self.dropDown)
    {
        return NO;
    }
    return YES;
}



@end
